@@ -1,7 +1,7 @@
 /*
  *	The PCI Utilities -- List All PCI Devices
  *
- *	Copyright (c) 1997--2005 Martin Mares <mj@ucw.cz>
+ *	Copyright (c) 1997--2006 Martin Mares <mj@ucw.cz>
  *
  *	Can be freely distributed and used under the terms of the GNU GPL.
  */
@@ -23,8 +23,9 @@ static struct pci_filter filter;	/* Device filter */
 static int show_tree;			/* Show bus tree */
 static int machine_readable;		/* Generate machine-readable output */
 static int map_mode;			/* Bus mapping mode enabled */
+static int show_domains;		/* Show domain numbers (0=disabled, 1=auto-detected, 2=requested) */
 
-static char options[] = "nvbxs:d:ti:mgM" GENERIC_OPTIONS ;
+static char options[] = "nvbxs:d:ti:mgMD" GENERIC_OPTIONS ;
 
 static char help_msg[] = "\
 Usage: lspci [<switches>]\n\
@@ -40,6 +41,7 @@ Usage: lspci [<switches>]\n\
 -t\t\tShow bus tree\n\
 -m\t\tProduce machine-readable output\n\
 -i <file>\tUse specified ID database instead of %s\n\
+-D\t\tAlways show domain numbers\n\
 -M\t\tEnable `bus mapping' mode (dangerous; root only)\n"
 GENERIC_HELP
 ;
@@ -104,6 +106,8 @@ scan_device(struct pci_dev *p)
 {
   struct device *d;
 
+  if (p->domain)
+    show_domains = 1;
   if (!pci_filter_match(&filter, p))
     return NULL;
   d = xmalloc(sizeof(struct device));
@@ -239,7 +243,7 @@ show_slot_name(struct device *d)
 {
   struct pci_dev *p = d->dev;
 
-  if (p->domain)
+  if (!machine_readable ? show_domains : (p->domain || show_domains >= 2))
     printf("%04x:", p->domain);
   printf("%02x:%02x.%d", p->bus, p->dev, p->func);
 }
@@ -1129,7 +1133,7 @@ static void show_express_link(struct device *d, int where, int type)
 	FLAG(w, PCI_EXP_LNKCTL_XSYNCH));
   w = get_conf_word(d, where + PCI_EXP_LNKSTA);
   printf("\t\tLink: Speed %s, Width x%d\n",
-	link_speed(t & PCI_EXP_LNKSTA_SPEED), (t & PCI_EXP_LNKSTA_WIDTH) >> 4);
+	link_speed(w & PCI_EXP_LNKSTA_SPEED), (w & PCI_EXP_LNKSTA_WIDTH) >> 4);
 }
 
 static const char *indicator(int code)
@@ -1676,7 +1680,7 @@ show_verbose(struct device *d)
 	      putchar(')');
 	    }
 	  if (cache_line)
-	    printf(", Cache Line Size %02x", cache_line);
+	    printf(", Cache Line Size: %d bytes", cache_line * 4);
 	  putchar('\n');
 	}
       if (int_pin || irq)
@@ -2317,6 +2321,9 @@ main(int argc, char **argv)
       case 'M':
 	map_mode++;
 	break;
+      case 'D':
+	show_domains = 2;
+	break;
       default:
 	if (parse_generic_option(i, pacc, optarg))
 	  break;
